import socket
import string
try:
    import Cookie
except ImportError:
    pass
import base64

import context

class SessionError(Exception):
    pass

# Simple session server
class SessionServerContextMixin(context.SessionBase):
    def __init__(self):
        context.SessionBase.__init__(self)
        self.__sesid = None
        
    def _get_sesid_from_cookie(self):
        hdr = self.request.get_header('Cookie')
        if hdr:
            c = Cookie.SimpleCookie(hdr)
            try:
                return c[self.app.ses_appid()].value
            except:
                pass

    def sesid(self):
        return self.__sesid

    def load_session(self):
        sesid = self._get_sesid_from_cookie()
        text = None
        if sesid:
            text = self.app.get_session(sesid)
        if not text:
            sesid = self.app.new_session()
        self.__sesid = sesid
        if text:
            text = base64.decodestring(text)
            try:
                self.decode_session(text)
            except:
                self.app.del_session(sesid)
                self.__sesid = self.app.new_session()
                
    def write_headers(self):
        c = Cookie.SimpleCookie()
        c[self.app.ses_appid()] = self.__sesid
        prefix_len = len('Set-Cookie: ')
        self.request.write_header('Set-Cookie', str(c)[prefix_len:])

    def save_session(self):
        text = self.encode_session()
        text = base64.encodestring(text)
        self.app.put_session(self.__sesid, text)

    def remove_session(self):
        context.SessionBase.remove_session(self)
        if self.__sesid is not None:
            self.app.del_session(self.__sesid)

class SessionServerAppMixin:
    def __init__(self, appid, server = 'localhost', port = 34343, age = 1800):
        self.__appid = appid
        self.__server = server
        self.__port = port
        self.__sock = None
        self.__age = age
        try:
            self.server_connect()
        except:
            pass

    def ses_appid(self):
        return self.__appid

    def server_connect(self):
        if self.__sock:
            return
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            sock.connect((self.__server, self.__port))
            self.__sock = sock
        except:
            raise SessionError('could not connect to server')

    def server_read(self):
        try:
            text = self.__sock.recv(16384)
            if text:
                return text
        except socket.error:
            pass
        self.__sock = None
        raise SessionError('server disconnect')

    def server_write(self, text):
        try:
            self.__sock.send(text)
            return
        except socket.error:
            pass
        self.__sock = None
        raise SessionError('server disconnect')

    def get_session(self, sesid):
        self.server_connect()
        self.server_write('get %s %s\r\n' % (self.__appid, sesid))
        text = ''
        while 1:
            text = text + self.server_read()
            pos = string.find(text, '\r\n')
            if pos >= 0:
                if text[:2] != 'OK':
                    return
                text = text[pos + 2:]
                break
        while 1:
            if text[-4:] == '\r\n\r\n':
                return text
            text = text + self.server_read()

    def new_session(self):
        self.server_connect()
        self.server_write('new %s %s\r\n' % (self.__appid, self.__age))
        resp = self.server_read()
        if resp[:3] == 'OK ':
            return string.rstrip(resp[3:])

    def put_session(self, sesid, text):
        self.server_connect()
        self.server_write('put %s %s\r\n' % (self.__appid, sesid))
        resp = self.server_read()
        if resp[:3] == 'OK ':
            self.server_write(string.replace(text, '\n', '\r\n'))
            self.server_write('\r\n')
        resp = self.server_read()

    def del_session(self, sesid):
        self.server_connect()
        self.server_write('del %s %s\r\n' % (self.__appid, sesid))
        self.server_read()
